@@ -4,7 +4,7 @@
 /*           http://hts-engine.sourceforge.net/                      */
 /* ----------------------------------------------------------------- */
 /*                                                                   */
-/*  Copyright (c) 2001-2014  Nagoya Institute of Technology          */
+/*  Copyright (c) 2001-2015  Nagoya Institute of Technology          */
 /*                           Department of Computer Science          */
 /*                                                                   */
 /*                2001-2008  Tokyo Institute of Technology           */
@@ -894,7 +894,7 @@ HTS_Boolean HTS_ModelSet_load(HTS_ModelSet * ms, char **voices, size_t num_voice
 {
    size_t i, j, k, s, e;
    HTS_Boolean error = FALSE;
-   HTS_File *fp;
+   HTS_File *fp = NULL;
    char buff1[HTS_MAXBUFLEN];
    char buff2[HTS_MAXBUFLEN];
    size_t matched_size;
@@ -1067,7 +1067,10 @@ HTS_Boolean HTS_ModelSet_load(HTS_ModelSet * ms, char **voices, size_t num_voice
          }
       }
       if (error != FALSE) {
-         HTS_fclose(fp);
+         if (fp != NULL) {
+            HTS_fclose(fp);
+            fp = NULL;
+         }
          break;
       }
       /* reset STREAM options */
@@ -1183,7 +1186,10 @@ HTS_Boolean HTS_ModelSet_load(HTS_ModelSet * ms, char **voices, size_t num_voice
          free(temp_option);
       }
       if (error != FALSE) {
-         HTS_fclose(fp);
+         if (fp != NULL) {
+            HTS_fclose(fp);
+            fp = NULL;
+         }
          break;
       }
       /* reset POSITION */
@@ -1365,6 +1371,7 @@ HTS_Boolean HTS_ModelSet_load(HTS_ModelSet * ms, char **voices, size_t num_voice
                HTS_fseek(fp, start_of_data, SEEK_SET);
             }
          }
+         HTS_Window_clear(&ms->window[j]);      /* if windows were loaded already, release them */
          if (HTS_Window_load(&ms->window[j], win_fp, num_windows[j]) != TRUE)
             error = TRUE;
          for (k = 0; k < num_windows[j]; k++)
@@ -1452,7 +1459,10 @@ HTS_Boolean HTS_ModelSet_load(HTS_ModelSet * ms, char **voices, size_t num_voice
             free(temp_gv_tree[j]);
       free(temp_gv_tree);
       /* fclose */
-      HTS_fclose(fp);
+      if (fp != NULL) {
+         HTS_fclose(fp);
+         fp = NULL;
+      }
       if (error != FALSE)
          break;
    }
@@ -1640,7 +1650,7 @@ void HTS_ModelSet_get_parameter_index(HTS_ModelSet * ms, size_t voice_index, siz
 }
 
 /* HTS_ModelSet_get_parameter: get parameter using interpolation weight */
-void HTS_ModelSet_get_parameter(HTS_ModelSet * ms, size_t stream_index, size_t state_index, const char *string, const double *iw, double *mean, double *vari, double *msd)
+void HTS_ModelSet_get_parameter(HTS_ModelSet * ms, size_t stream_index, size_t state_index, const char *string, const double *const *iw, double *mean, double *vari, double *msd)
 {
    size_t i;
    size_t len = ms->stream[0][stream_index].vector_length * ms->stream[0][stream_index].num_windows;
@@ -1653,8 +1663,8 @@ void HTS_ModelSet_get_parameter(HTS_ModelSet * ms, size_t stream_index, size_t s
       *msd = 0.0;
 
    for (i = 0; i < ms->num_voices; i++)
-      if (iw[i] != 0.0)
-         HTS_Model_add_parameter(&ms->stream[i][stream_index], state_index, string, mean, vari, msd, iw[i]);
+      if (iw[i][stream_index] != 0.0)
+         HTS_Model_add_parameter(&ms->stream[i][stream_index], state_index, string, mean, vari, msd, iw[i][stream_index]);
 }
 
 /* HTS_ModelSet_get_gv_index: get gv PDF & tree index */
@@ -1664,7 +1674,7 @@ void HTS_ModelSet_get_gv_index(HTS_ModelSet * ms, size_t voice_index, size_t str
 }
 
 /* HTS_ModelSet_get_gv: get GV using interpolation weight */
-void HTS_ModelSet_get_gv(HTS_ModelSet * ms, size_t stream_index, const char *string, const double *iw, double *mean, double *vari)
+void HTS_ModelSet_get_gv(HTS_ModelSet * ms, size_t stream_index, const char *string, const double *const *iw, double *mean, double *vari)
 {
    size_t i;
    size_t len = ms->stream[0][stream_index].vector_length;
@@ -1674,8 +1684,8 @@ void HTS_ModelSet_get_gv(HTS_ModelSet * ms, size_t stream_index, const char *str
       vari[i] = 0.0;
    }
    for (i = 0; i < ms->num_voices; i++)
-      if (iw[i] != 0.0)
-         HTS_Model_add_parameter(&ms->gv[i][stream_index], 2, string, mean, vari, NULL, iw[i]);
+      if (iw[i][stream_index] != 0.0)
+         HTS_Model_add_parameter(&ms->gv[i][stream_index], 2, string, mean, vari, NULL, iw[i][stream_index]);
 }
 
 HTS_MODEL_C_END;
